# coding: utf-8

"""
    Jaqpot API

    A modern RESTful API for model management and prediction services, built using Spring Boot and Kotlin. Supports seamless integration with machine learning workflows.

    The version of the OpenAPI document: 1.0.0
    Contact: upci.ntua@gmail.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from jaqpotpy.api.openapi.models.error_code import ErrorCode

class TestErrorCode(unittest.TestCase):
    """ErrorCode unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testErrorCode(self):
        """Test ErrorCode"""
        # inst = ErrorCode()

if __name__ == '__main__':
    unittest.main()
