# coding: utf-8

"""
    Jaqpot API

    A modern RESTful API for model management and prediction services, built using Spring Boot and Kotlin. Supports seamless integration with machine learning workflows.

    The version of the OpenAPI document: 1.0.0
    Contact: upci.ntua@gmail.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from jaqpotpy.api.openapi.api.dataset_api import DatasetApi


class TestDatasetApi(unittest.TestCase):
    """DatasetApi unit test stubs"""

    def setUp(self) -> None:
        self.api = DatasetApi()

    def tearDown(self) -> None:
        pass

    def test_get_dataset_by_id(self) -> None:
        """Test case for get_dataset_by_id

        Get a Dataset
        """
        pass

    def test_get_datasets(self) -> None:
        """Test case for get_datasets

        Get Datasets by User ID
        """
        pass


if __name__ == '__main__':
    unittest.main()
