# coding: utf-8

"""
    Jaqpot API

    A modern RESTful API for model management and prediction services, built using Spring Boot and Kotlin. Supports seamless integration with machine learning workflows.

    The version of the OpenAPI document: 1.0.0
    Contact: upci.ntua@gmail.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from jaqpotpy.api.openapi.models.partially_update_model_feature_request import PartiallyUpdateModelFeatureRequest

class TestPartiallyUpdateModelFeatureRequest(unittest.TestCase):
    """PartiallyUpdateModelFeatureRequest unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> PartiallyUpdateModelFeatureRequest:
        """Test PartiallyUpdateModelFeatureRequest
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `PartiallyUpdateModelFeatureRequest`
        """
        model = PartiallyUpdateModelFeatureRequest()
        if include_optional:
            return PartiallyUpdateModelFeatureRequest(
                name = 'Updated Feature Name',
                units = 'mg/L',
                description = 'An updated description for this feature',
                feature_type = 'FLOAT',
                possible_values = [
                    jaqpotpy.api.openapi.models.feature_possible_value.FeaturePossibleValue(
                        key = 'value', 
                        value = 'value', )
                    ]
            )
        else:
            return PartiallyUpdateModelFeatureRequest(
                name = 'Updated Feature Name',
                feature_type = 'FLOAT',
        )
        """

    def testPartiallyUpdateModelFeatureRequest(self):
        """Test PartiallyUpdateModelFeatureRequest"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()
