# coding: utf-8

"""
    Jaqpot API

    A modern RESTful API for model management and prediction services, built using Spring Boot and Kotlin. Supports seamless integration with machine learning workflows.

    The version of the OpenAPI document: 1.0.0
    Contact: upci.ntua@gmail.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from jaqpotpy.api.openapi.api.organization_api import OrganizationApi


class TestOrganizationApi(unittest.TestCase):
    """OrganizationApi unit test stubs"""

    def setUp(self) -> None:
        self.api = OrganizationApi()

    def tearDown(self) -> None:
        pass

    def test_create_organization(self) -> None:
        """Test case for create_organization

        Create a new organization
        """
        pass

    def test_get_all_organizations_by_user(self) -> None:
        """Test case for get_all_organizations_by_user

        Get all user organizations
        """
        pass

    def test_get_all_organizations_for_user(self) -> None:
        """Test case for get_all_organizations_for_user

        Get all organizations for a specific user
        """
        pass

    def test_get_organization_by_name(self) -> None:
        """Test case for get_organization_by_name

        Get organization by name
        """
        pass

    def test_partial_update_organization(self) -> None:
        """Test case for partial_update_organization

        Partially update an existing organization
        """
        pass


if __name__ == '__main__':
    unittest.main()
