import os

"""
Configuration variables for Jaqpotpy
"""

"""
x -> Test variable
verbose -> Verbose outputs. eg descriptor creation, model training
global_seed -> Seed for various descriptors and models
version -> Jaqpotpy version
"""

x = 0
verbose = True
global_seed = 42
# version = jaqpotpy.__version__
